# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: metrics.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='metrics.proto',
  package='metrics',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\rmetrics.proto\x12\x07metrics\"N\n\rUploadRequest\x12*\n\x07windows\x18\x01 \x03(\x0b\x32\x19.metrics.PredictionWindow\x12\x11\n\tupload_ts\x18\x02 \x01(\x04\"\x10\n\x0eUploadResponse\"p\n\rWindowMessage\x12\x0f\n\x07\x64\x61ta_id\x18\x01 \x01(\t\x12\x11\n\twindow_id\x18\x02 \x01(\t\x12\x10\n\x08model_id\x18\x03 \x01(\t\x12)\n\x06window\x18\x04 \x01(\x0b\x32\x19.metrics.PredictionWindow\"\x93\x01\n\x06Rollup\x12\x11\n\trollup_id\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x04\x12\x12\n\nresolution\x18\x03 \x01(\x04\x12\x13\n\x0btime_bucket\x18\x04 \x01(\x04\x12\x11\n\texpire_at\x18\x05 \x01(\x04\x12)\n\x06window\x18\x06 \x01(\x0b\x32\x19.metrics.PredictionWindow\"\xff\x01\n\x10PredictionWindow\x12%\n\x05model\x18\x02 \x01(\x0b\x32\x16.metrics.DeployedModel\x12\x17\n\x0fnum_predictions\x18\x03 \x01(\x04\x12\x10\n\x08start_ts\x18\x04 \x01(\x04\x12\x0e\n\x06\x65nd_ts\x18\x05 \x01(\x04\x12@\n\x0c\x64\x61ta_streams\x18\x06 \x03(\x0b\x32*.metrics.PredictionWindow.DataStreamsEntry\x1aG\n\x10\x44\x61taStreamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.metrics.DataStream:\x02\x38\x01\"\x91\x01\n\rDeployedModel\x12\x17\n\x0f\x64\x65ployment_name\x18\x02 \x01(\t\x12\x36\n\x08metadata\x18\x03 \x03(\x0b\x32$.metrics.DeployedModel.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9f\x03\n\nDataStream\x12\x33\n\x0b\x64\x61ta_source\x18\x01 \x01(\x0e\x32\x1e.metrics.DataStream.DataSource\x12/\n\tdata_type\x18\x02 \x01(\x0e\x32\x1c.metrics.DataStream.DataType\x12\x31\n\x07metrics\x18\x03 \x03(\x0b\x32 .metrics.DataStream.MetricsEntry\x1a\x43\n\x0cMetricsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.metrics.DataMetric:\x02\x38\x01\"\\\n\nDataSource\x12\x13\n\x0fNOT_INITIALIZED\x10\x00\x12\x0f\n\x0bMODEL_INPUT\x10\x01\x12\x10\n\x0cMODEL_OUTPUT\x10\x02\x12\x16\n\x12\x46\x45\x41TURE_IMPORTANCE\x10\x03\"U\n\x08\x44\x61taType\x12\x11\n\rNOT_SPECIFIED\x10\x00\x12\x0b\n\x07TABULAR\x10\x01\x12\t\n\x05IMAGE\x10\x02\x12\x08\n\x04TEXT\x10\x03\x12\t\n\x05\x41UDIO\x10\x04\x12\t\n\x05VIDEO\x10\x05\"\xdd\x03\n\nDataMetric\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x37\n\ndimensions\x18\x02 \x03(\x0b\x32#.metrics.DataMetric.DimensionsEntry\x12,\n\x04type\x18\x03 \x01(\x0e\x32\x1e.metrics.DataMetric.MetricType\x12*\n\x0bgauge_value\x18\x04 \x01(\x0b\x32\x13.metrics.GaugeValueH\x00\x12.\n\rcounter_value\x18\x05 \x01(\x0b\x32\x15.metrics.CounterValueH\x00\x12*\n\x0bratio_value\x18\x06 \x01(\x0b\x32\x13.metrics.RatioValueH\x00\x12\x38\n\x12\x64istribution_value\x18\x07 \x01(\x0b\x32\x1a.metrics.DistributionValueH\x00\x1a\x31\n\x0f\x44imensionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"V\n\nMetricType\x12\x13\n\x0fNOT_INITIALIZED\x10\x00\x12\t\n\x05GAUGE\x10\x01\x12\x0b\n\x07\x43OUNTER\x10\x02\x12\t\n\x05RATIO\x10\x03\x12\x10\n\x0c\x44ISTRIBUTION\x10\x04\x42\r\n\x0bvalue_oneof\"\x1b\n\nGaugeValue\x12\r\n\x05gauge\x18\x01 \x01(\x01\"\x1f\n\x0c\x43ounterValue\x12\x0f\n\x07\x63ounter\x18\x01 \x01(\x01\",\n\nRatioValue\x12\x0f\n\x07\x63ounter\x18\x01 \x01(\x01\x12\r\n\x05total\x18\x02 \x01(\x01\"\xba\x01\n\x11\x44istributionValue\x12:\n\x0bsketch_impl\x18\x01 \x01(\x0e\x32%.metrics.DistributionValue.SketchImpl\x12,\n\x0csketch_kll10\x18\x02 \x01(\x0b\x32\x14.metrics.SketchKLL10H\x00\"*\n\nSketchImpl\x12\x11\n\rNOT_SPECIFIED\x10\x00\x12\t\n\x05KLL10\x10\x01\x42\x0f\n\rone_of_sketch\"\xa4\x02\n\x0bSketchKLL10\x12\t\n\x01k\x18\x01 \x01(\x04\x12\t\n\x01\x63\x18\x02 \x01(\x01\x12\t\n\x01H\x18\x03 \x01(\x04\x12\x0c\n\x04size\x18\x04 \x01(\x04\x12\x10\n\x08max_size\x18\x05 \x01(\x04\x12\x30\n\titem_type\x18\x06 \x01(\x0e\x32\x1d.metrics.SketchKLL10.ItemType\x12>\n\x11\x63ompactors_double\x18\x07 \x03(\x0b\x32#.metrics.SketchKLL10CompactorDouble\x12>\n\x11\x63ompactors_string\x18\x08 \x03(\x0b\x32#.metrics.SketchKLL10CompactorString\"\"\n\x08ItemType\x12\n\n\x06\x44OUBLE\x10\x00\x12\n\n\x06STRING\x10\x01\"+\n\x1aSketchKLL10CompactorDouble\x12\r\n\x05items\x18\x01 \x03(\x01\"+\n\x1aSketchKLL10CompactorString\x12\r\n\x05items\x18\x01 \x03(\tb\x06proto3'
)



_DATASTREAM_DATASOURCE = _descriptor.EnumDescriptor(
  name='DataSource',
  full_name='metrics.DataStream.DataSource',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOT_INITIALIZED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MODEL_INPUT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MODEL_OUTPUT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FEATURE_IMPORTANCE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1031,
  serialized_end=1123,
)
_sym_db.RegisterEnumDescriptor(_DATASTREAM_DATASOURCE)

_DATASTREAM_DATATYPE = _descriptor.EnumDescriptor(
  name='DataType',
  full_name='metrics.DataStream.DataType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOT_SPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TABULAR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IMAGE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUDIO', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VIDEO', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1125,
  serialized_end=1210,
)
_sym_db.RegisterEnumDescriptor(_DATASTREAM_DATATYPE)

_DATAMETRIC_METRICTYPE = _descriptor.EnumDescriptor(
  name='MetricType',
  full_name='metrics.DataMetric.MetricType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOT_INITIALIZED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GAUGE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COUNTER', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RATIO', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DISTRIBUTION', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1589,
  serialized_end=1675,
)
_sym_db.RegisterEnumDescriptor(_DATAMETRIC_METRICTYPE)

_DISTRIBUTIONVALUE_SKETCHIMPL = _descriptor.EnumDescriptor(
  name='SketchImpl',
  full_name='metrics.DistributionValue.SketchImpl',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOT_SPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='KLL10', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1928,
  serialized_end=1970,
)
_sym_db.RegisterEnumDescriptor(_DISTRIBUTIONVALUE_SKETCHIMPL)

_SKETCHKLL10_ITEMTYPE = _descriptor.EnumDescriptor(
  name='ItemType',
  full_name='metrics.SketchKLL10.ItemType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DOUBLE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2248,
  serialized_end=2282,
)
_sym_db.RegisterEnumDescriptor(_SKETCHKLL10_ITEMTYPE)


_UPLOADREQUEST = _descriptor.Descriptor(
  name='UploadRequest',
  full_name='metrics.UploadRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='windows', full_name='metrics.UploadRequest.windows', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='upload_ts', full_name='metrics.UploadRequest.upload_ts', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=104,
)


_UPLOADRESPONSE = _descriptor.Descriptor(
  name='UploadResponse',
  full_name='metrics.UploadResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=122,
)


_WINDOWMESSAGE = _descriptor.Descriptor(
  name='WindowMessage',
  full_name='metrics.WindowMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_id', full_name='metrics.WindowMessage.data_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='window_id', full_name='metrics.WindowMessage.window_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_id', full_name='metrics.WindowMessage.model_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='window', full_name='metrics.WindowMessage.window', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=124,
  serialized_end=236,
)


_ROLLUP = _descriptor.Descriptor(
  name='Rollup',
  full_name='metrics.Rollup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rollup_id', full_name='metrics.Rollup.rollup_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='metrics.Rollup.version', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resolution', full_name='metrics.Rollup.resolution', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_bucket', full_name='metrics.Rollup.time_bucket', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expire_at', full_name='metrics.Rollup.expire_at', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='window', full_name='metrics.Rollup.window', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=239,
  serialized_end=386,
)


_PREDICTIONWINDOW_DATASTREAMSENTRY = _descriptor.Descriptor(
  name='DataStreamsEntry',
  full_name='metrics.PredictionWindow.DataStreamsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='metrics.PredictionWindow.DataStreamsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='metrics.PredictionWindow.DataStreamsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=573,
  serialized_end=644,
)

_PREDICTIONWINDOW = _descriptor.Descriptor(
  name='PredictionWindow',
  full_name='metrics.PredictionWindow',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model', full_name='metrics.PredictionWindow.model', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_predictions', full_name='metrics.PredictionWindow.num_predictions', index=1,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_ts', full_name='metrics.PredictionWindow.start_ts', index=2,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_ts', full_name='metrics.PredictionWindow.end_ts', index=3,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_streams', full_name='metrics.PredictionWindow.data_streams', index=4,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_PREDICTIONWINDOW_DATASTREAMSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=389,
  serialized_end=644,
)


_DEPLOYEDMODEL_METADATAENTRY = _descriptor.Descriptor(
  name='MetadataEntry',
  full_name='metrics.DeployedModel.MetadataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='metrics.DeployedModel.MetadataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='metrics.DeployedModel.MetadataEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=745,
  serialized_end=792,
)

_DEPLOYEDMODEL = _descriptor.Descriptor(
  name='DeployedModel',
  full_name='metrics.DeployedModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deployment_name', full_name='metrics.DeployedModel.deployment_name', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='metrics.DeployedModel.metadata', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYEDMODEL_METADATAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=647,
  serialized_end=792,
)


_DATASTREAM_METRICSENTRY = _descriptor.Descriptor(
  name='MetricsEntry',
  full_name='metrics.DataStream.MetricsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='metrics.DataStream.MetricsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='metrics.DataStream.MetricsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=962,
  serialized_end=1029,
)

_DATASTREAM = _descriptor.Descriptor(
  name='DataStream',
  full_name='metrics.DataStream',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_source', full_name='metrics.DataStream.data_source', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_type', full_name='metrics.DataStream.data_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metrics', full_name='metrics.DataStream.metrics', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DATASTREAM_METRICSENTRY, ],
  enum_types=[
    _DATASTREAM_DATASOURCE,
    _DATASTREAM_DATATYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=795,
  serialized_end=1210,
)


_DATAMETRIC_DIMENSIONSENTRY = _descriptor.Descriptor(
  name='DimensionsEntry',
  full_name='metrics.DataMetric.DimensionsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='metrics.DataMetric.DimensionsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='metrics.DataMetric.DimensionsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1538,
  serialized_end=1587,
)

_DATAMETRIC = _descriptor.Descriptor(
  name='DataMetric',
  full_name='metrics.DataMetric',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='metrics.DataMetric.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dimensions', full_name='metrics.DataMetric.dimensions', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='metrics.DataMetric.type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gauge_value', full_name='metrics.DataMetric.gauge_value', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='counter_value', full_name='metrics.DataMetric.counter_value', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ratio_value', full_name='metrics.DataMetric.ratio_value', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='distribution_value', full_name='metrics.DataMetric.distribution_value', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DATAMETRIC_DIMENSIONSENTRY, ],
  enum_types=[
    _DATAMETRIC_METRICTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value_oneof', full_name='metrics.DataMetric.value_oneof',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1213,
  serialized_end=1690,
)


_GAUGEVALUE = _descriptor.Descriptor(
  name='GaugeValue',
  full_name='metrics.GaugeValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='gauge', full_name='metrics.GaugeValue.gauge', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1692,
  serialized_end=1719,
)


_COUNTERVALUE = _descriptor.Descriptor(
  name='CounterValue',
  full_name='metrics.CounterValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='counter', full_name='metrics.CounterValue.counter', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1721,
  serialized_end=1752,
)


_RATIOVALUE = _descriptor.Descriptor(
  name='RatioValue',
  full_name='metrics.RatioValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='counter', full_name='metrics.RatioValue.counter', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='total', full_name='metrics.RatioValue.total', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1754,
  serialized_end=1798,
)


_DISTRIBUTIONVALUE = _descriptor.Descriptor(
  name='DistributionValue',
  full_name='metrics.DistributionValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sketch_impl', full_name='metrics.DistributionValue.sketch_impl', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sketch_kll10', full_name='metrics.DistributionValue.sketch_kll10', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DISTRIBUTIONVALUE_SKETCHIMPL,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='one_of_sketch', full_name='metrics.DistributionValue.one_of_sketch',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1801,
  serialized_end=1987,
)


_SKETCHKLL10 = _descriptor.Descriptor(
  name='SketchKLL10',
  full_name='metrics.SketchKLL10',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='k', full_name='metrics.SketchKLL10.k', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='c', full_name='metrics.SketchKLL10.c', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='H', full_name='metrics.SketchKLL10.H', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='metrics.SketchKLL10.size', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_size', full_name='metrics.SketchKLL10.max_size', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='item_type', full_name='metrics.SketchKLL10.item_type', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='compactors_double', full_name='metrics.SketchKLL10.compactors_double', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='compactors_string', full_name='metrics.SketchKLL10.compactors_string', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SKETCHKLL10_ITEMTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1990,
  serialized_end=2282,
)


_SKETCHKLL10COMPACTORDOUBLE = _descriptor.Descriptor(
  name='SketchKLL10CompactorDouble',
  full_name='metrics.SketchKLL10CompactorDouble',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='metrics.SketchKLL10CompactorDouble.items', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2284,
  serialized_end=2327,
)


_SKETCHKLL10COMPACTORSTRING = _descriptor.Descriptor(
  name='SketchKLL10CompactorString',
  full_name='metrics.SketchKLL10CompactorString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='metrics.SketchKLL10CompactorString.items', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2329,
  serialized_end=2372,
)

_UPLOADREQUEST.fields_by_name['windows'].message_type = _PREDICTIONWINDOW
_WINDOWMESSAGE.fields_by_name['window'].message_type = _PREDICTIONWINDOW
_ROLLUP.fields_by_name['window'].message_type = _PREDICTIONWINDOW
_PREDICTIONWINDOW_DATASTREAMSENTRY.fields_by_name['value'].message_type = _DATASTREAM
_PREDICTIONWINDOW_DATASTREAMSENTRY.containing_type = _PREDICTIONWINDOW
_PREDICTIONWINDOW.fields_by_name['model'].message_type = _DEPLOYEDMODEL
_PREDICTIONWINDOW.fields_by_name['data_streams'].message_type = _PREDICTIONWINDOW_DATASTREAMSENTRY
_DEPLOYEDMODEL_METADATAENTRY.containing_type = _DEPLOYEDMODEL
_DEPLOYEDMODEL.fields_by_name['metadata'].message_type = _DEPLOYEDMODEL_METADATAENTRY
_DATASTREAM_METRICSENTRY.fields_by_name['value'].message_type = _DATAMETRIC
_DATASTREAM_METRICSENTRY.containing_type = _DATASTREAM
_DATASTREAM.fields_by_name['data_source'].enum_type = _DATASTREAM_DATASOURCE
_DATASTREAM.fields_by_name['data_type'].enum_type = _DATASTREAM_DATATYPE
_DATASTREAM.fields_by_name['metrics'].message_type = _DATASTREAM_METRICSENTRY
_DATASTREAM_DATASOURCE.containing_type = _DATASTREAM
_DATASTREAM_DATATYPE.containing_type = _DATASTREAM
_DATAMETRIC_DIMENSIONSENTRY.containing_type = _DATAMETRIC
_DATAMETRIC.fields_by_name['dimensions'].message_type = _DATAMETRIC_DIMENSIONSENTRY
_DATAMETRIC.fields_by_name['type'].enum_type = _DATAMETRIC_METRICTYPE
_DATAMETRIC.fields_by_name['gauge_value'].message_type = _GAUGEVALUE
_DATAMETRIC.fields_by_name['counter_value'].message_type = _COUNTERVALUE
_DATAMETRIC.fields_by_name['ratio_value'].message_type = _RATIOVALUE
_DATAMETRIC.fields_by_name['distribution_value'].message_type = _DISTRIBUTIONVALUE
_DATAMETRIC_METRICTYPE.containing_type = _DATAMETRIC
_DATAMETRIC.oneofs_by_name['value_oneof'].fields.append(
  _DATAMETRIC.fields_by_name['gauge_value'])
_DATAMETRIC.fields_by_name['gauge_value'].containing_oneof = _DATAMETRIC.oneofs_by_name['value_oneof']
_DATAMETRIC.oneofs_by_name['value_oneof'].fields.append(
  _DATAMETRIC.fields_by_name['counter_value'])
_DATAMETRIC.fields_by_name['counter_value'].containing_oneof = _DATAMETRIC.oneofs_by_name['value_oneof']
_DATAMETRIC.oneofs_by_name['value_oneof'].fields.append(
  _DATAMETRIC.fields_by_name['ratio_value'])
_DATAMETRIC.fields_by_name['ratio_value'].containing_oneof = _DATAMETRIC.oneofs_by_name['value_oneof']
_DATAMETRIC.oneofs_by_name['value_oneof'].fields.append(
  _DATAMETRIC.fields_by_name['distribution_value'])
_DATAMETRIC.fields_by_name['distribution_value'].containing_oneof = _DATAMETRIC.oneofs_by_name['value_oneof']
_DISTRIBUTIONVALUE.fields_by_name['sketch_impl'].enum_type = _DISTRIBUTIONVALUE_SKETCHIMPL
_DISTRIBUTIONVALUE.fields_by_name['sketch_kll10'].message_type = _SKETCHKLL10
_DISTRIBUTIONVALUE_SKETCHIMPL.containing_type = _DISTRIBUTIONVALUE
_DISTRIBUTIONVALUE.oneofs_by_name['one_of_sketch'].fields.append(
  _DISTRIBUTIONVALUE.fields_by_name['sketch_kll10'])
_DISTRIBUTIONVALUE.fields_by_name['sketch_kll10'].containing_oneof = _DISTRIBUTIONVALUE.oneofs_by_name['one_of_sketch']
_SKETCHKLL10.fields_by_name['item_type'].enum_type = _SKETCHKLL10_ITEMTYPE
_SKETCHKLL10.fields_by_name['compactors_double'].message_type = _SKETCHKLL10COMPACTORDOUBLE
_SKETCHKLL10.fields_by_name['compactors_string'].message_type = _SKETCHKLL10COMPACTORSTRING
_SKETCHKLL10_ITEMTYPE.containing_type = _SKETCHKLL10
DESCRIPTOR.message_types_by_name['UploadRequest'] = _UPLOADREQUEST
DESCRIPTOR.message_types_by_name['UploadResponse'] = _UPLOADRESPONSE
DESCRIPTOR.message_types_by_name['WindowMessage'] = _WINDOWMESSAGE
DESCRIPTOR.message_types_by_name['Rollup'] = _ROLLUP
DESCRIPTOR.message_types_by_name['PredictionWindow'] = _PREDICTIONWINDOW
DESCRIPTOR.message_types_by_name['DeployedModel'] = _DEPLOYEDMODEL
DESCRIPTOR.message_types_by_name['DataStream'] = _DATASTREAM
DESCRIPTOR.message_types_by_name['DataMetric'] = _DATAMETRIC
DESCRIPTOR.message_types_by_name['GaugeValue'] = _GAUGEVALUE
DESCRIPTOR.message_types_by_name['CounterValue'] = _COUNTERVALUE
DESCRIPTOR.message_types_by_name['RatioValue'] = _RATIOVALUE
DESCRIPTOR.message_types_by_name['DistributionValue'] = _DISTRIBUTIONVALUE
DESCRIPTOR.message_types_by_name['SketchKLL10'] = _SKETCHKLL10
DESCRIPTOR.message_types_by_name['SketchKLL10CompactorDouble'] = _SKETCHKLL10COMPACTORDOUBLE
DESCRIPTOR.message_types_by_name['SketchKLL10CompactorString'] = _SKETCHKLL10COMPACTORSTRING
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UploadRequest = _reflection.GeneratedProtocolMessageType('UploadRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADREQUEST,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.UploadRequest)
  })
_sym_db.RegisterMessage(UploadRequest)

UploadResponse = _reflection.GeneratedProtocolMessageType('UploadResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADRESPONSE,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.UploadResponse)
  })
_sym_db.RegisterMessage(UploadResponse)

WindowMessage = _reflection.GeneratedProtocolMessageType('WindowMessage', (_message.Message,), {
  'DESCRIPTOR' : _WINDOWMESSAGE,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.WindowMessage)
  })
_sym_db.RegisterMessage(WindowMessage)

Rollup = _reflection.GeneratedProtocolMessageType('Rollup', (_message.Message,), {
  'DESCRIPTOR' : _ROLLUP,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.Rollup)
  })
_sym_db.RegisterMessage(Rollup)

PredictionWindow = _reflection.GeneratedProtocolMessageType('PredictionWindow', (_message.Message,), {

  'DataStreamsEntry' : _reflection.GeneratedProtocolMessageType('DataStreamsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PREDICTIONWINDOW_DATASTREAMSENTRY,
    '__module__' : 'metrics_pb2'
    # @@protoc_insertion_point(class_scope:metrics.PredictionWindow.DataStreamsEntry)
    })
  ,
  'DESCRIPTOR' : _PREDICTIONWINDOW,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.PredictionWindow)
  })
_sym_db.RegisterMessage(PredictionWindow)
_sym_db.RegisterMessage(PredictionWindow.DataStreamsEntry)

DeployedModel = _reflection.GeneratedProtocolMessageType('DeployedModel', (_message.Message,), {

  'MetadataEntry' : _reflection.GeneratedProtocolMessageType('MetadataEntry', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYEDMODEL_METADATAENTRY,
    '__module__' : 'metrics_pb2'
    # @@protoc_insertion_point(class_scope:metrics.DeployedModel.MetadataEntry)
    })
  ,
  'DESCRIPTOR' : _DEPLOYEDMODEL,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.DeployedModel)
  })
_sym_db.RegisterMessage(DeployedModel)
_sym_db.RegisterMessage(DeployedModel.MetadataEntry)

DataStream = _reflection.GeneratedProtocolMessageType('DataStream', (_message.Message,), {

  'MetricsEntry' : _reflection.GeneratedProtocolMessageType('MetricsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASTREAM_METRICSENTRY,
    '__module__' : 'metrics_pb2'
    # @@protoc_insertion_point(class_scope:metrics.DataStream.MetricsEntry)
    })
  ,
  'DESCRIPTOR' : _DATASTREAM,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.DataStream)
  })
_sym_db.RegisterMessage(DataStream)
_sym_db.RegisterMessage(DataStream.MetricsEntry)

DataMetric = _reflection.GeneratedProtocolMessageType('DataMetric', (_message.Message,), {

  'DimensionsEntry' : _reflection.GeneratedProtocolMessageType('DimensionsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATAMETRIC_DIMENSIONSENTRY,
    '__module__' : 'metrics_pb2'
    # @@protoc_insertion_point(class_scope:metrics.DataMetric.DimensionsEntry)
    })
  ,
  'DESCRIPTOR' : _DATAMETRIC,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.DataMetric)
  })
_sym_db.RegisterMessage(DataMetric)
_sym_db.RegisterMessage(DataMetric.DimensionsEntry)

GaugeValue = _reflection.GeneratedProtocolMessageType('GaugeValue', (_message.Message,), {
  'DESCRIPTOR' : _GAUGEVALUE,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.GaugeValue)
  })
_sym_db.RegisterMessage(GaugeValue)

CounterValue = _reflection.GeneratedProtocolMessageType('CounterValue', (_message.Message,), {
  'DESCRIPTOR' : _COUNTERVALUE,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.CounterValue)
  })
_sym_db.RegisterMessage(CounterValue)

RatioValue = _reflection.GeneratedProtocolMessageType('RatioValue', (_message.Message,), {
  'DESCRIPTOR' : _RATIOVALUE,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.RatioValue)
  })
_sym_db.RegisterMessage(RatioValue)

DistributionValue = _reflection.GeneratedProtocolMessageType('DistributionValue', (_message.Message,), {
  'DESCRIPTOR' : _DISTRIBUTIONVALUE,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.DistributionValue)
  })
_sym_db.RegisterMessage(DistributionValue)

SketchKLL10 = _reflection.GeneratedProtocolMessageType('SketchKLL10', (_message.Message,), {
  'DESCRIPTOR' : _SKETCHKLL10,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.SketchKLL10)
  })
_sym_db.RegisterMessage(SketchKLL10)

SketchKLL10CompactorDouble = _reflection.GeneratedProtocolMessageType('SketchKLL10CompactorDouble', (_message.Message,), {
  'DESCRIPTOR' : _SKETCHKLL10COMPACTORDOUBLE,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.SketchKLL10CompactorDouble)
  })
_sym_db.RegisterMessage(SketchKLL10CompactorDouble)

SketchKLL10CompactorString = _reflection.GeneratedProtocolMessageType('SketchKLL10CompactorString', (_message.Message,), {
  'DESCRIPTOR' : _SKETCHKLL10COMPACTORSTRING,
  '__module__' : 'metrics_pb2'
  # @@protoc_insertion_point(class_scope:metrics.SketchKLL10CompactorString)
  })
_sym_db.RegisterMessage(SketchKLL10CompactorString)


_PREDICTIONWINDOW_DATASTREAMSENTRY._options = None
_DEPLOYEDMODEL_METADATAENTRY._options = None
_DATASTREAM_METRICSENTRY._options = None
_DATAMETRIC_DIMENSIONSENTRY._options = None
# @@protoc_insertion_point(module_scope)



import re
import unicodedata
import time
import logging
import numpy as np
import pandas as pd

import graphsignal
from graphsignal.predictions import Prediction
from graphsignal.metrics import *
from graphsignal import metrics_pb2

logger = logging.getLogger('graphsignal')
_rand = np.random.RandomState(int(time.time()))

MAX_COLUMNS = 250


def estimate_size(data):
    if data is None:
        return 0

    if isinstance(data, list):
        return len(data)
    elif isinstance(data, dict):
        if len(data) > 0:
            first_elem = next(iter(data.values()))
            if isinstance(first_elem, list):
                return len(first_elem)
            elif isinstance(first_elem, np.ndarray):
                return first_elem.shape[0] if first_elem.ndim > 0 else 1
            else:
                return 1
    elif isinstance(data, np.ndarray):
        return data.shape[0] if data.ndim > 0 else 1
    elif isinstance(data, pd.DataFrame):
        return data.shape[0]
    return 0


def update_metrics(metric_updaters, window_proto, prediction_buffer):
    if len(prediction_buffer) == 0:
        return

    start_ts = time.time()
    input_buffer = [p.input_data
                    for p in prediction_buffer if p.input_data is not None]
    input_columns = prediction_buffer[-1].input_columns
    input_columns, input_arr = _convert_to_numpy(
        input_buffer, columns=input_columns)

    output_buffer = [p.output_data
                     for p in prediction_buffer if p.output_data is not None]
    output_columns = prediction_buffer[-1].output_columns
    output_columns, output_arr = _convert_to_numpy(
        output_buffer, columns=output_columns)

    if input_arr is None and output_arr is None:
        return

    if input_arr is not None:
        data_stream = get_data_stream(
            window_proto,
            metrics_pb2.DataStream.DataSource.MODEL_INPUT,
            metrics_pb2.DataStream.DataType.TABULAR)
        _update_tabular_metrics(
            metric_updaters,
            data_stream,
            input_columns,
            input_arr)

    if output_arr is not None:
        data_stream = get_data_stream(
            window_proto,
            metrics_pb2.DataStream.DataSource.MODEL_OUTPUT,
            metrics_pb2.DataStream.DataType.TABULAR)
        _update_tabular_metrics(
            metric_updaters,
            data_stream,
            output_columns,
            output_arr)

    logger.debug('Computing data metrics took %.3f sec',
                 time.time() - start_ts)


def _update_tabular_metrics(metric_updaters, data_stream_proto, columns, data):
    if len(columns) != len(data):
        raise ValueError(
            'Error processing tabular data: columns and values do not match')

    instance_count = data[0].shape[0]
    metric_updater = get_metric_updater(
        metric_updaters, data_stream_proto, 'instance_count')
    metric_updater.update_counter(instance_count)

    metric_updater = get_metric_updater(
        metric_updaters, data_stream_proto, 'column_count')
    metric_updater.update_gauge(len(columns))

    for column_name, column_values in zip(
            columns[:MAX_COLUMNS], data[:MAX_COLUMNS]):
        if isinstance(column_name, str) and column_name.lower() in (
                'timestamp', 'time', 'datetime', 'date'):
            continue
        column_type = column_values.dtype
        if np.issubdtype(column_type, np.datetime64):
            continue
        value_count = column_values.shape[0]

        dimensions = {
            'column': str(column_name)[:50]
        }

        if np.issubdtype(column_type, np.number):
            missing_count = np.count_nonzero(np.isnan(column_values))
            missing_count += np.count_nonzero(np.isinf(column_values))
            metric_updater = get_metric_updater(
                metric_updaters, data_stream_proto, 'missing_values', dimensions)
            metric_updater.update_ratio(missing_count, value_count)

            zero_count = np.count_nonzero(np.equal(column_values, 0))
            metric_updater = get_metric_updater(
                metric_updaters, data_stream_proto, 'zero_values', dimensions)
            metric_updater.update_ratio(zero_count, value_count)

            finite_column_values = column_values[np.isfinite(column_values)]
            finite_value_count = finite_column_values.shape[0]

            if np.issubdtype(column_type, np.integer):
                integer_count = finite_value_count
                float_count = 0
            elif np.issubdtype(column_type, np.floating):
                integer_count = np.count_nonzero(
                    np.equal(np.mod(finite_column_values, 1), 0))
                float_count = finite_value_count - integer_count
            else:
                integer_count = 0
                float_count = 0

            metric_updater = get_metric_updater(
                metric_updaters, data_stream_proto, 'integer_values', dimensions)
            metric_updater.update_ratio(integer_count, value_count)

            metric_updater = get_metric_updater(
                metric_updaters, data_stream_proto, 'float_values', dimensions)
            metric_updater.update_ratio(float_count, value_count)

            if finite_column_values.shape[0] > 0:
                finite_column_values_list = finite_column_values.tolist()

                metric_updater = get_metric_updater(
                    metric_updaters, data_stream_proto, 'distribution', dimensions)
                metric_updater.update_distribution(finite_column_values_list)
        else:
            string_column_values = [
                m for m in column_values.tolist() if isinstance(m, str)]
            if len(string_column_values) > 0:
                missing_count = value_count - len(string_column_values)
                metric_updater = get_metric_updater(
                    metric_updaters, data_stream_proto, 'missing_values', dimensions)
                metric_updater.update_ratio(missing_count, value_count)

                empty_count = string_column_values.count('')
                metric_updater = get_metric_updater(
                    metric_updaters, data_stream_proto, 'empty_values', dimensions)
                metric_updater.update_ratio(empty_count, value_count)

                string_count = len(string_column_values)
                metric_updater = get_metric_updater(
                    metric_updaters, data_stream_proto, 'string_values', dimensions)
                metric_updater.update_ratio(string_count, value_count)

                metric_updater = get_metric_updater(
                    metric_updaters, data_stream_proto, 'distribution', dimensions)
                metric_updater.update_distribution(
                    _truncate_strings(string_column_values))


def _convert_to_numpy(data_buffer, columns=None):
    if len(data_buffer) == 0:
        return None, None

    data = None

    first_item = data_buffer[0]

    # list
    if isinstance(first_item, list):
        keys = list(range(len(first_item)))
        data_index = {key: [] for key in keys}
        for item in data_buffer:
            for key in keys:
                data_index[key].append(item[key])
        if columns is None:
            columns = [str(key) for key in keys]
        data = []
        for arrays in data_index.values():
            flat_array = _flatten_and_convert_arrays(arrays)
            if flat_array is None:
                data = None
                break
            data.append(flat_array)

    # dict
    elif isinstance(first_item, dict):
        keys = list(first_item.keys())
        data_index = {key: [] for key in keys}
        for item in data_buffer:
            for key in keys:
                data_index[key].append(item[key])
        columns = [str(key) for key in keys]
        data = []
        for arrays in data_index.values():
            flat_array = _flatten_and_convert_arrays(arrays)
            if flat_array is None:
                data = None
                break
            data.append(flat_array)

    # numpy.ndarray
    elif isinstance(first_item, np.ndarray):
        if first_item.ndim == 2:
            if columns is None:
                columns = [str(column)
                           for column in range(first_item.shape[1])]
            data_arr = np.concatenate(data_buffer)
            data = [
                col_arr.ravel() for col_arr in np.hsplit(
                    data_arr, data_arr.shape[1])]

    # pandas.DataFrame
    elif isinstance(first_item, pd.DataFrame):
        data_df = pd.concat(data_buffer, ignore_index=True)
        columns = data_df.columns.values.tolist()
        data = []
        for column in columns:
            data.append(data_df[column].to_numpy())
        columns = [str(column) for column in columns]

    if data is None:
        raise ValueError(
            'Unsupported data format: please use one or two-dimensional list, dict, numpy.  ndarray or pandas.DataFrame')

    return columns, data


def _flatten_and_convert_arrays(arrays):
    flat_array = None
    first_array = arrays[0]
    if isinstance(first_array, list):
        flat_array = np.asarray([elem for array in arrays for elem in array])
    elif isinstance(first_array, np.ndarray):
        if first_array.ndim == 1:
            flat_array = np.concatenate(arrays)
    else:
        flat_array = np.asarray(arrays)  # scalars

    return flat_array


def _is_2d(data):
    if isinstance(data, list):
        if len(data) > 0:
            if not _is_1d(data[0]):
                return False
        return True
    elif isinstance(data, dict):
        if len(data) > 0:
            if not _is_1d(list(data.values())[0]):
                return False
        return True
    elif isinstance(data, np.ndarray):
        return data.ndim == 2
    return False


def _is_1d(data):
    if isinstance(data, list):
        if len(data) > 0:
            if not _is_scalar(data[0]):
                return False
        return True
    if isinstance(data, dict):
        if len(data) > 0:
            if not _is_scalar(list(data.values())[0]):
                return False
        return True
    elif isinstance(data, np.ndarray):
        return data.ndim == 1
    return False


def _is_scalar(data):
    return isinstance(data, (int, float, str))


def _truncate_strings(values, max_size=18, front_size=10, tail_size=5):
    truncated_values = []
    for value in values:
        if len(value) > max_size:
            truncated_values.append(
                value[:front_size] + '...' + value[-tail_size:])
        else:
            truncated_values.append(value)
    return truncated_values



import unittest
import logging
from unittest.mock import patch, Mock
import sys
import numpy as np
import time
import pandas as pd

import graphsignal
from graphsignal import metrics_pb2
from graphsignal.metrics import *
from graphsignal.sketches.kll import KLLSketch

logger = logging.getLogger('graphsignal')


class MetricsTest(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        if len(logger.handlers) == 0:
            logger.addHandler(logging.StreamHandler(sys.stdout))
        graphsignal.configure(api_key='k1', debug_mode=True)

    def tearDown(self):
        graphsignal.shutdown()

    def test_update_gauge(self):
        window = metrics_pb2.PredictionWindow()
        metric_updater = get_metric_updater(
            {}, window.data_streams['1'], 'test')

        metric_updater.update_gauge(1.1)
        metric_updater.update_gauge(1.2)
        self.assertEqual(metric_updater._metric.gauge_value.gauge, 1.2)

    def test_update_counter(self):
        window = metrics_pb2.PredictionWindow()
        metric_updater = get_metric_updater(
            {}, window.data_streams['1'], 'test')

        metric_updater.update_counter(4)
        metric_updater.update_counter(2.01)
        self.assertEqual(
            metric_updater._metric.counter_value.counter, 6.01)

    def test_update_ratio(self):
        window = metrics_pb2.PredictionWindow()
        metric_updater = get_metric_updater(
            {}, window.data_streams['1'], 'test')

        metric_updater.update_ratio(0, 5)
        metric_updater.update_ratio(1, 30)
        self.assertEqual(
            metric_updater._metric.ratio_value.counter, 1)
        self.assertEqual(
            metric_updater._metric.ratio_value.total, 35)

    def test_update_distribution(self):
        window = metrics_pb2.PredictionWindow()
        metric_updater = get_metric_updater(
            {}, window.data_streams['1'], 'test')

        data = [1.1, 1.1, 2, 2, 3, 3, 3, 4.0001, 4.0001]
        metric_updater.update_distribution(data)

        data = [1.1, 5000]
        metric_updater.update_distribution(data)

        metric_updater.finalize()

        self.assertEqual(metric_updater._sketch.count(), 11)

        k2 = KLLSketch()
        k2.from_proto(metric_updater._metric.distribution_value.sketch_kll10)
        self.assertEqual(k2.count(), 11)



import copy
import logging
import math
import time
import hashlib
import random
import numpy as np

from graphsignal.sketches.kll import KLLSketch

logger = logging.getLogger('graphsignal')


def get_data_stream(window_proto, data_source, data_type):
    data_stream = window_proto.data_streams[str(data_source)]
    if data_stream.data_source == data_stream.DataSource.NOT_INITIALIZED:
        data_stream.data_source = data_source
        data_stream.data_type = data_type
    return data_stream


def get_metric_updater(
        metric_updaters, data_stream_proto, name, dimensions=None):
    metric_id = _sha1('{0}:{1}:{2}'.format(
        data_stream_proto.data_source,
        name,
        str(dict(sorted(dimensions.items()))) if dimensions is not None else ''),
        size=12)

    if metric_id not in metric_updaters:
        metric = metric_updaters[metric_id] = DataMetricUpdater(
            data_stream_proto.metrics[metric_id], name, dimensions)
        return metric
    else:
        return metric_updaters[metric_id]


class DataMetricUpdater(object):
    __slots__ = [
        '_metric',
        '_sketch'
    ]

    def __init__(self, metric, name, dimensions=None):
        self._metric = metric
        self._sketch = None

        self._metric.name = name
        if dimensions is not None:
            for name, value in dimensions.items():
                self._metric.dimensions[name] = value

    def update_gauge(self, value):
        if self._metric.type == self._metric.NOT_INITIALIZED:
            self._metric.type = self._metric.GAUGE
        self._metric.gauge_value.gauge = value

    def update_counter(self, value):
        if self._metric.type == self._metric.NOT_INITIALIZED:
            self._metric.type = self._metric.COUNTER
            self._metric.counter_value.counter = 0
        self._metric.counter_value.counter += value

    def update_ratio(self, value, total):
        if self._metric.type == self._metric.NOT_INITIALIZED:
            self._metric.type = self._metric.RATIO
            self._metric.ratio_value.counter = 0
            self._metric.ratio_value.total = 0
        self._metric.ratio_value.counter += value
        self._metric.ratio_value.total += total

    def update_distribution(self, values):
        if len(values) == 0:
            return

        if self._metric.type == self._metric.NOT_INITIALIZED:
            self._metric.type = self._metric.DISTRIBUTION
            self._metric.distribution_value.sketch_impl = self._metric.distribution_value.KLL10
            k_val = 128 if isinstance(values[0], (int, float)) else 10
            self._sketch = KLLSketch(k=k_val)

        for value in values:
            self._sketch.update(value)

    def finalize(self):
        if self._metric.type == self._metric.DISTRIBUTION:
            self._sketch.to_proto(self._metric.distribution_value.sketch_kll10)


def _sha1(text, size=-1):
    sha1_hash = hashlib.sha1()
    sha1_hash.update(text.encode('utf-8'))
    return sha1_hash.hexdigest()[0:size]



import math
import random

# Implements https://arxiv.org/pdf/1603.05346v2.pdf


class KLLSketch:
    __slots__ = [
        '_k',
        '_c',
        '_compactors',
        '_H',
        '_size',
        '_max_size'
    ]

    def __init__(self, k=128):
        if k <= 0:
            raise ValueError("invalid k")

        self._k = k
        self._c = 2.0 / 3.0
        self._compactors = []
        self._H = 0
        self._size = 0
        self._max_size = 0

        self._grow()

    def _capacity(self, h):
        return int(math.ceil(self._k * self._c ** (self._H - h - 1))) + 1

    def _grow(self):
        self._compactors.append(Compactor())
        self._H = len(self._compactors)
        self._max_size = sum([self._capacity(h) for h in range(self._H)])

    def _compact(self):
        for h in range(len(self._compactors)):
            if len(self._compactors[h]) >= self._capacity(h):
                if h + 1 >= self._H:
                    self._grow()

                self._size -= (len(self._compactors[h]) +
                               len(self._compactors[h + 1]))
                self._compactors[h + 1].extend(self._compactors[h].compact())
                self._size += (len(self._compactors[h]) +
                               len(self._compactors[h + 1]))
                break

    def update(self, item):
        self._compactors[0].append(item)
        self._size += 1
        if self._size >= self._max_size:
            self._compact()

    def merge(self, other):
        while self._H < other._H:
            self._grow()

        for h in range(other._H):
            self._compactors[h].extend(other._compactors[h])
            self._size += len(other._compactors[h])

        while self._size >= self._max_size:
            self._compact()

    def count(self):
        count = 0
        for (h, items) in enumerate(self._compactors):
            count += len(items) * 2**h
        return count

    def distribution(self):
        weights = []
        for (h, items) in enumerate(self._compactors):
            weights.extend([(item, 2**h) for item in items])
        weights.sort()

        dist = []
        for item, weight in weights:
            if len(dist) > 0 and dist[-1][0] == item:
                dist[-1][1] += weight
            else:
                dist.append([item, weight])
        return dist
        
    def ranks(self):
        weights = []
        for (h, items) in enumerate(self._compactors):
            weights.extend([(item, 2**h) for item in items])

        weights.sort()

        cum_weight = 0
        pairs = []
        for (item, weight) in weights:
            cum_weight += weight
            pairs.append((item, cum_weight))
        return pairs

    def cdf(self):
        weights = []
        for (h, items) in enumerate(self._compactors):
            weights.extend([(item, 2**h) for item in items])
        total_weight = sum([weight for (item, weight) in weights])

        weights.sort()

        cum_weight = 0
        pairs = []
        for (item, weight) in weights:
            cum_weight += weight
            pairs.append((item, float(cum_weight) / float(total_weight)))

        return pairs

    def to_proto(self, proto):
        proto.k = self._k
        proto.c = self._c
        proto.H = self._H
        proto.size = self._size
        proto.max_size = self._max_size
        if isinstance(self._compactors[-1][0], (int, float)):
            proto.item_type = proto.ItemType.DOUBLE
            for (h, items) in enumerate(self._compactors):
                proto.compactors_double.add().items[:] = items
        elif isinstance(self._compactors[-1][0], str):
            proto.item_type = proto.ItemType.STRING
            for (h, items) in enumerate(self._compactors):
                proto.compactors_string.add().items[:] = items

    def from_proto(self, proto):
        self._k = proto.k
        self._c = proto.c
        self._H = proto.H
        self._size = proto.size
        self._max_size = proto.max_size
        self._compactors = []
        if proto.item_type == proto.ItemType.DOUBLE:
            for (h, compactor_proto) in enumerate(proto.compactors_double):
                compactor = Compactor()
                compactor[:] = compactor_proto.items
                self._compactors.append(compactor)
        elif proto.item_type == proto.ItemType.STRING:
            for (h, compactor_proto) in enumerate(proto.compactors_string):
                compactor = Compactor()
                compactor[:] = compactor_proto.items
                self._compactors.append(compactor)


class Compactor(list):
    def __init__(self):
        pass

    def compact(self):
        self.sort()

        offset = int(random.random() < 0.5)
        keep = self[offset::2]
        self[:] = self[-1:] if len(self) % 2 == 1 else []

        return keep



import logging
import time

logger = logging.getLogger('graphsignal')


class Prediction(object):
    __slots__ = [
        'input_data',
        'input_columns',
        'output_data',
        'output_columns'
    ]

    def __init__(
            self,
            input_data=None,
            input_columns=None,
            output_data=None,
            output_columns=None):
        self.input_data = input_data
        self.input_columns = input_columns
        self.output_data = output_data
        self.output_columns = output_columns



import unittest
import logging
import time
from unittest.mock import patch, Mock
import sys
import pprint
import numpy as np

import graphsignal
from graphsignal.uploader import Uploader
from graphsignal import metrics_pb2

logger = logging.getLogger('graphsignal')


class SessionsTest(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        if len(logger.handlers) == 0:
            logger.addHandler(logging.StreamHandler(sys.stdout))
        graphsignal.sessions.reset_all()
        graphsignal.configure(api_key='k1', debug_mode=True)

    def tearDown(self):
        graphsignal.sessions.reset_all()
        graphsignal.shutdown()

    @patch.object(Uploader, 'flush')
    @patch.object(Uploader, 'upload_window')
    @patch('graphsignal.statistics.update_metrics')
    def test_prediction_batched(self, mocked_update_metrics,
                                mocked_upload_window, mocked_flush):
        session = graphsignal.session('d1')
        session.set_metadata(key='k1', value='v1')
        session.log_prediction(
            input_data=[[1, 2], [3, 4]],
            input_columns=['A', 'B'],
            output_data=[5, 6])

        session._upload_window(force=True)

        mocked_upload_window.assert_called_once()

        uploaded_window = mocked_upload_window.call_args[0][0]
        self.assertEqual(uploaded_window.num_predictions, 2)
        self.assertEqual(uploaded_window.model.metadata['k1'], 'v1')

    @patch.object(Uploader, 'flush')
    @patch.object(Uploader, 'upload_window')
    def test_prediction_not_uploaded(self, mocked_upload_window, mocked_flush):
        session = graphsignal.session(deployment_name='d1')

        session.log_prediction(input_data=[[1, 2], [3, 4]])
        session.log_prediction(input_data=[[1, 2], [3, 4]])

        self.assertEqual(session._current_window.num_predictions, 4)
        mocked_upload_window.assert_not_called()

    @patch.object(Uploader, 'flush')
    @patch.object(Uploader, 'upload_window')
    def test_prediction_uploaded(
            self, mocked_upload_window, mocked_flush):
        session = graphsignal.session('d1')

        session.log_prediction(input_data=[[1, 2], [3, 4]])
        session._current_window.start_ts = session._current_window.start_ts - \
            graphsignal._get_config().window_seconds - 1
        session.log_prediction(input_data=[[1, 2], [3, 4]])

        mocked_upload_window.assert_called_once()



import time
import logging
import threading
import sys
import traceback
import platform
from google.protobuf.json_format import MessageToDict

import graphsignal
from graphsignal import statistics
from graphsignal.uploader import Uploader
from graphsignal.predictions import Prediction
from graphsignal import metrics_pb2

logger = logging.getLogger('graphsignal')

MAX_METADATA_SIZE = 10

MAX_BUFFER_SIZE = 100
MIN_WINDOW_length = 600

_session_index = {}
_session_index_lock = threading.Lock()


class Session(object):
    __slots__ = [
        '_deployment_name',
        '_metadata',
        '_current_window',
        '_prediction_buffer',
        '_buffer_size',
        '_metric_updaters',
        '_update_lock',
        '_is_updated',
        '_upload_timer'
    ]

    def __init__(self, deployment_name):
        self._deployment_name = deployment_name
        self._metadata = {}
        self._update_lock = threading.Lock()
        self._reset_current_window()
        self._current_window.model.deployment_name = deployment_name

    def _reset_current_window(self):
        self._current_window = metrics_pb2.PredictionWindow()
        self._prediction_buffer = []
        self._buffer_size = 0
        self._metric_updaters = {}
        self._is_updated = False

    def _set_updated(self):
        self._is_updated = True
        if self._upload_window():
            graphsignal._get_uploader().flush_in_thread()

    def set_metadata(self, key=None, value=None):
        '''
        Set model metadata.

        Args:
            key (:obj:`str`):
                Metadata entry key.
            value (:obj:`str` or :obj:`int` or :obj:`float`):
                Metadata entry value.
        '''

        if not isinstance(key, str) or len(key) > 250:
            logger.error('invalid metadata entry key format')
            return
        if not isinstance(value, (str, int, float)) or len(str(value)) > 2500:
            logger.error(
                'invalid metadata entry value format for key: {0}'.format(key))
            return

        if len(self._current_window.model.metadata) >= MAX_METADATA_SIZE:
            logger.error(
                'too many metadata entries, max={0}'.format(MAX_METADATA_SIZE))
            return

        self._metadata[key] = str(value)

    def log_prediction(
            self,
            input_data=None,
            input_columns=None,
            output_data=None,
            output_columns=None,
            actual_timestamp=None):
        '''
        Log single or batch model prediction.

        See `Supported Data Formats <https://graphsignal.com/docs/integrations/python/supported-data-formats/>`_
        for detailed description of data types and formats.

        Computed data statistics are computed and uploaded for time windows at certain intervals and on process exit.
        No raw data is uploaded.

        Args:
            input_data (:obj:`list` or :obj:`dict` or :obj:`numpy.ndarray` or :obj:`pandas.DataFrame`, optional):
                Input data instances.
            input_columns (:obj:`list`, optional):
                A list of input data column names. If not provided, column names are inferred from `input_data`.
            output_data (:obj:`list` or :obj:`dict` or :obj:`numpy.ndarray` or :obj:`pandas.DataFrame`, optional):
                Output data instances.
            output_columns (:obj:`list`, optional):
                A list of output data column names. If not provided, column names are inferred from `output_data`.
            actual_timestamp (:obj:`int`, optional, default is current timestamp):
                Actual unix timestamp of the measurement, when different from current timestamp.
        '''

        data_size = max(
            statistics.estimate_size(input_data),
            statistics.estimate_size(output_data))
        if data_size == 0:
            logger.debug('Logged empty data')
            return

        self._buffer_size += data_size
        self._current_window.num_predictions += data_size

        timestamp = actual_timestamp if actual_timestamp is not None else int(
            time.time())
        if self._current_window.start_ts == 0:
            self._current_window.start_ts = timestamp
        self._current_window.end_ts = timestamp

        with self._update_lock:
            self._prediction_buffer.append(Prediction(
                input_data=input_data,
                input_columns=input_columns,
                output_data=output_data,
                output_columns=output_columns))

        if self._buffer_size > graphsignal._get_config().buffer_size:
            self._merge_buffer()

        self._set_updated()

    def _merge_buffer(self):
        if self._buffer_size > 0:
            with self._update_lock:
                current_buffer = self._prediction_buffer
                self._prediction_buffer = []
                self._buffer_size = 0
            try:
                statistics.update_metrics(
                    self._metric_updaters, self._current_window, current_buffer)
            except BaseException:
                logger.error('Error updating metrics', exc_info=True)

    def _upload_window(self, force=False):
        if not self._is_updated:
            return False

        if not force:
            if self._current_window.end_ts - \
                    self._current_window.start_ts < graphsignal._get_config().window_seconds:
                return False

        self._merge_buffer()
        for metric_updater in self._metric_updaters.values():
            metric_updater.finalize()

        self._current_window.model.deployment_name = self._deployment_name
        for key, value in self._metadata.items():
            self._current_window.model.metadata[key] = value

        with self._update_lock:
            window = self._current_window
            self._reset_current_window()

        if logger.isEnabledFor(logging.DEBUG):
            logger.debug('Uploading window:')
            logger.debug(MessageToDict(window))

        graphsignal._get_uploader().upload_window(window)
        return True


def get_session(deployment_name):
    if not deployment_name or len(deployment_name) > 250:
        raise ValueError('invalid deployment_name format')

    with _session_index_lock:
        if deployment_name in _session_index:
            return _session_index[deployment_name]
        else:
            sess = Session(deployment_name)
            _session_index[deployment_name] = sess
            return sess


def reset_all():
    with _session_index_lock:
        _session_index.clear()


def upload_all(force=False):
    session_list = None
    with _session_index_lock:
        session_list = _session_index.values()

    uploaded = False
    for session in session_list:
        if session._upload_window(force=force):
            uploaded = True

    return uploaded



import time
import sys
import os
import logging
import threading
import uuid
import atexit

from graphsignal import version
from graphsignal import sessions
from graphsignal.uploader import Uploader

logger = logging.getLogger('graphsignal')

UPLOAD_INTERVAL = 120

_config = None
_uploader = None


class Config(object):
    def __init__(self):
        self.api_key = None
        self.debug_mode = None
        self.window_seconds = None
        self.buffer_size = None


def _get_config():
    return _config


def _get_uploader():
    return _uploader


def configure(api_key, debug_mode=False, window_seconds=600, buffer_size=100):
    '''
    Configures and initializes the logger.

    Args:
        api_key (:obj:`str`):
            The access key for communication with the Graphsignal servers.
        debug_mode (:obj:`bool`, optional):
            Enable/disable debug output.
        window_seconds (:obj:`int`, optional, default 600):
            The length of prediction time windows for which data statistics
            are reported.
        buffer_size (:obj:`int`, optional, default 100):
            The maximum mumber of model input and/or output data instances
            kept in memory before computing statistics.
    '''

    global _config, _uploader

    if _config:
        logger.warning('Logger already configured')
        return

    if debug_mode:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.WARNING)

    if not api_key:
        logger.error('Missing argument: api_key')
        return

    _config = Config()
    _config.api_key = api_key
    _config.debug_mode = debug_mode
    _config.window_seconds = window_seconds
    _config.buffer_size = buffer_size

    _uploader = Uploader()
    _uploader.configure()

    sessions.reset_all()

    atexit.register(shutdown)

    logger.debug('Logger configured')


def shutdown():
    '''
    Send any collected data to the server and cleanup.

    Normally, when python scripts exists, this method is automatically called. Use this method,
    if you want to explicitely shutdown the logger.
    '''

    global _config

    if not _config:
        logger.error(
            'Logger not configured, please use graphsignal.configure()')
        return

    atexit.unregister(shutdown)

    sessions.upload_all(force=True)
    _uploader.flush()

    _config = None

    logger.debug('Logger shutdown')


def tick():
    '''
    Check if any data can be uploaded and if yes, upload.
    Can be used in situations when logging is irregular and/or rare.
    '''

    if not _config:
        logger.error(
            'Logger not configured, please use graphsignal.configure()')
        return

    sessions.upload_all()
    _uploader.flush()


def session(deployment_name):
    '''
    Get logging session for the model identified by model deployment name.

    Args:
        deployment_name (:obj:`str`, optional):
            Model deployment name, e.g. `model1_production`, `modelB_canary` or any other string value.
    Returns:
        :obj:`Session` - session object for logging prediction data.
    Raises:
        `ValueError`: When `deployment_name` is invalid.
    '''

    if not _config:
        raise ValueError(
            'graphsignal not configured, please use graphsignal.configure()')

    return sessions.get_session(deployment_name)



import time
import unittest
import logging
from unittest.mock import patch, Mock
import sys
import numpy as np
import pandas as pd
import pprint
from google.protobuf.json_format import MessageToDict

import graphsignal
from graphsignal import statistics
from graphsignal.predictions import Prediction
from graphsignal import metrics_pb2

logger = logging.getLogger('graphsignal')


class StatisticsTest(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        if len(logger.handlers) == 0:
            logger.addHandler(logging.StreamHandler(sys.stdout))
        graphsignal.configure(api_key='k1', debug_mode=True)

    def tearDown(self):
        graphsignal.shutdown()

    def test_data_size(self):
        self.assertEqual(statistics.estimate_size([[1], [2]]), 2)
        self.assertEqual(statistics.estimate_size(np.array([[1], [2]])), 2)
        self.assertEqual(statistics.estimate_size(
            {'a': [1, 2, 3], 'b': [4, 5, 6]}), 3)
        self.assertEqual(statistics.estimate_size(
            {'a': np.asarray([1, 2, 3]), 'b': np.asarray([4, 5, 6])}), 3)
        self.assertEqual(
            statistics.estimate_size(
                pd.DataFrame(
                    data=[
                        [1],
                        [2]])),
            2)

    def test_convert_to_numpy_dict_of_scalars(self):
        d1 = {'c1': 1, 'c2': 2.1}
        d2 = {'c1': 3, 'c2': 5.1}
        columns, data = statistics._convert_to_numpy([d1, d2])
        self.assertIsNotNone(data)
        self.assertEqual(columns, ['c1', 'c2'])
        self.assertTrue(np.array_equal(
            data[0], np.asarray([1, 3])))
        self.assertTrue(np.array_equal(
            data[1], np.asarray([2.1, 5.1])))

    def test_convert_to_numpy_dict_of_lists(self):
        d1 = {'c1': [1, 2], 'c2': [3.1, 4.1]}
        d2 = {'c1': [3], 'c2': [5.1]}
        columns, data = statistics._convert_to_numpy([d1, d2])
        self.assertIsNotNone(data)
        self.assertEqual(columns, ['c1', 'c2'])
        self.assertTrue(np.array_equal(
            data[0], np.asarray([1, 2, 3])))
        self.assertTrue(np.array_equal(
            data[1], np.asarray([3.1, 4.1, 5.1])))
        self.assertEqual(data[0].dtype, 'int64')
        self.assertEqual(data[1].dtype, 'float64')

    def test_convert_to_numpy_list_of_ndarray(self):
        d1 = [np.asarray([1, 2]), np.asarray([3.1, 4.1])]
        d2 = [np.asarray([3, 4]), np.asarray([5.1, 6.1])]
        columns, data = statistics._convert_to_numpy(
            [d1, d2], columns=['A', 'B'])
        self.assertIsNotNone(data)
        self.assertEqual(columns, ['A', 'B'])
        self.assertTrue(np.array_equal(
            data[0], np.asarray([1, 2, 3, 4])))
        self.assertTrue(np.array_equal(
            data[1], np.asarray([3.1, 4.1, 5.1, 6.1])))

    def test_convert_to_numpy_dict_of_ndarray(self):
        d1 = {'c1': np.asarray([1, 2]), 'c2': np.asarray([3.1, 4.1])}
        d2 = {'c1': np.asarray([5]), 'c2': np.asarray([6.1])}
        columns, data = statistics._convert_to_numpy([d1, d2])
        self.assertIsNotNone(data)
        self.assertEqual(columns, ['c1', 'c2'])
        self.assertTrue(np.array_equal(
            data[0], np.asarray([1, 2, 5])))
        self.assertTrue(np.array_equal(
            data[1], np.asarray([3.1, 4.1, 6.1])))

    def test_convert_to_numpy_dataframe_with_columns(self):
        d1 = pd.DataFrame(data={'c1': [1, 2], 'c2': [3.1, 4.1]})
        d2 = pd.DataFrame(data={'c1': [3], 'c2': [5.1]})
        columns, data = statistics._convert_to_numpy([d1, d2])
        self.assertIsNotNone(data)
        self.assertEqual(columns, ['c1', 'c2'])
        self.assertTrue(np.array_equal(
            data[0], np.asarray([1, 2, 3])))
        self.assertTrue(np.array_equal(
            data[1], np.asarray([3.1, 4.1, 5.1])))
        self.assertEqual(data[0].dtype, 'int64')
        self.assertEqual(data[1].dtype, 'float64')

    def test_convert_to_numpy_dataframe_wo_columns(self):
        d = pd.DataFrame(data=[[1, 3.1], [2, 4.1]])
        columns, data = statistics._convert_to_numpy([d])
        self.assertIsNotNone(data)
        self.assertEqual(columns, ['0', '1'])
        self.assertTrue(np.array_equal(
            data[0], np.asarray([1, 2])))
        self.assertTrue(np.array_equal(
            data[1], np.asarray([3.1, 4.1])))

    def test_convert_to_numpy_ndarray(self):
        d1 = np.asarray([[1, 3.1], [2, 4.1]])
        d2 = np.asarray([[8, 12]])
        columns, data = statistics._convert_to_numpy([d1, d2])
        self.assertIsNotNone(data)
        self.assertEqual(columns, ['0', '1'])
        self.assertTrue(np.array_equal(
            data[0], np.asarray([1, 2, 8])))
        self.assertTrue(np.array_equal(
            data[1], np.asarray([3.1, 4.1, 12])))
        self.assertEqual(data[0].dtype, 'float64')
        self.assertEqual(data[1].dtype, 'float64')

    def test_convert_to_numpy_ndarray_3d(self):
        d = np.full((2, 5, 3), [1, 2, 3.3])
        with self.assertRaises(ValueError):
            statistics._convert_to_numpy([d])

    def test_update_metrics_empty(self):
        window = metrics_pb2.PredictionWindow()
        statistics.update_metrics({}, window, [])

        self.assertEqual(len(window.data_streams[str(
            metrics_pb2.DataStream.DataSource.MODEL_INPUT)].metrics), 0)
        self.assertEqual(len(window.data_streams[str(
            metrics_pb2.DataStream.DataSource.MODEL_OUTPUT)].metrics), 0)
        self.assertEqual(len(window.data_streams[str(
            metrics_pb2.DataStream.DataSource.FEATURE_IMPORTANCE)].metrics), 0)

    def test_update_metrics(self):
        window = metrics_pb2.PredictionWindow()
        metric_updaters = {}

        input_data = {
            'f1': [1, 1, 2, 0],
            'f2': [3.5, 4.5, 5.5, 0],
            'f3': ['a', 'b', 'c', 'c'],
            'f4': [0, float('nan'), float('inf'), 2]}
        output_data = [0.1, 0.2, 0.1, 0.4]

        statistics.update_metrics(
            metric_updaters, window, [
                Prediction(
                    input_data=input_data, output_data=output_data)])

        for metric_updater in metric_updaters.values():
            metric_updater.finalize()

        window_dict = MessageToDict(window)

        input_metrics_json = window_dict['dataStreams'][str(
            metrics_pb2.DataStream.DataSource.MODEL_INPUT)]['metrics']
        output_metrics_json = window_dict['dataStreams'][str(
            metrics_pb2.DataStream.DataSource.MODEL_OUTPUT)]['metrics']

        for metric in input_metrics_json.values():
            if 'distributionValue' in metric:
                del metric['distributionValue']['sketchKll10']

        for metric in output_metrics_json.values():
            if 'distributionValue' in metric:
                del metric['distributionValue']['sketchKll10']

        #pp = pprint.PrettyPrinter()
        #pp.pprint(input_metrics_json)
        #pp.pprint(output_metrics_json)

        self.assertEqual(input_metrics_json,
                         {'1c359883cf3d': {'dimensions': {'column': 'f1'},
                                           'distributionValue': {'sketchImpl': 'KLL10'},
                                           'name': 'distribution',
                                           'type': 'DISTRIBUTION'},
                             '3aa8ce8a745e': {'counterValue': {'counter': 4.0},
                                              'name': 'instance_count',
                                              'type': 'COUNTER'},
                             '401c985323db': {'dimensions': {'column': 'f1'},
                                              'name': 'missing_values',
                                              'ratioValue': {'total': 4.0},
                                              'type': 'RATIO'},
                             '43689dc15d24': {'dimensions': {'column': 'f1'},
                                              'name': 'integer_values',
                                              'ratioValue': {'counter': 4.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             '60258f4378d5': {'dimensions': {'column': 'f4'},
                                              'name': 'missing_values',
                                              'ratioValue': {'counter': 2.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             '682f9d56838c': {'dimensions': {'column': 'f4'},
                                              'name': 'integer_values',
                                              'ratioValue': {'counter': 2.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             '6add4609813c': {'gaugeValue': {'gauge': 4.0},
                                              'name': 'column_count',
                                              'type': 'GAUGE'},
                             '6c077d9e5a61': {'dimensions': {'column': 'f4'},
                                              'name': 'zero_values',
                                              'ratioValue': {'counter': 1.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             '9cac7145da6a': {'dimensions': {'column': 'f2'},
                                              'name': 'missing_values',
                                              'ratioValue': {'total': 4.0},
                                              'type': 'RATIO'},
                             '9df17486f062': {'dimensions': {'column': 'f3'},
                                              'name': 'missing_values',
                                              'ratioValue': {'total': 4.0},
                                              'type': 'RATIO'},
                             'a0e8a20c8920': {'dimensions': {'column': 'f3'},
                                              'name': 'empty_values',
                                              'ratioValue': {'total': 4.0},
                                              'type': 'RATIO'},
                             'b930a73df792': {'dimensions': {'column': 'f1'},
                                              'name': 'zero_values',
                                              'ratioValue': {'counter': 1.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             'cff27a80f0b7': {'dimensions': {'column': 'f1'},
                                              'name': 'float_values',
                                              'ratioValue': {'total': 4.0},
                                              'type': 'RATIO'},
                             'd96d26ea23b1': {'dimensions': {'column': 'f4'},
                                              'distributionValue': {'sketchImpl': 'KLL10'},
                                              'name': 'distribution',
                                              'type': 'DISTRIBUTION'},
                             'dd7c7096ed49': {'dimensions': {'column': 'f4'},
                                              'name': 'float_values',
                                              'ratioValue': {'total': 4.0},
                                              'type': 'RATIO'},
                             'e5073f3ae895': {'dimensions': {'column': 'f2'},
                                              'name': 'integer_values',
                                              'ratioValue': {'counter': 1.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             'e55eece578e6': {'dimensions': {'column': 'f3'},
                                              'name': 'string_values',
                                              'ratioValue': {'counter': 4.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             'ebad37241fb2': {'dimensions': {'column': 'f2'},
                                              'name': 'float_values',
                                              'ratioValue': {'counter': 3.0, 'total': 4.0},
                                              'type': 'RATIO'},
                             'f0c86a2c6f9d': {'dimensions': {'column': 'f3'},
                                              'distributionValue': {'sketchImpl': 'KLL10'},
                                              'name': 'distribution',
                                              'type': 'DISTRIBUTION'},
                             'f8179a7753dc': {'dimensions': {'column': 'f2'},
                                              'distributionValue': {'sketchImpl': 'KLL10'},
                                              'name': 'distribution',
                                              'type': 'DISTRIBUTION'},
                             'ff8da228450b': {'dimensions': {'column': 'f2'},
                                              'name': 'zero_values',
                                              'ratioValue': {'counter': 1.0, 'total': 4.0},
                                              'type': 'RATIO'}})
        self.assertEqual(output_metrics_json,
                         {'05c94b6ef3dd': {'dimensions': {'column': '3'},
                                           'name': 'missing_values',
                                           'ratioValue': {'total': 1.0},
                                           'type': 'RATIO'},
                             '08bdcbcb78e5': {'dimensions': {'column': '1'},
                                              'name': 'float_values',
                                              'ratioValue': {'counter': 1.0, 'total': 1.0},
                                              'type': 'RATIO'},
                             '094ce9a486c0': {'gaugeValue': {'gauge': 4.0},
                                              'name': 'column_count',
                                              'type': 'GAUGE'},
                             '0a030c2aa2b1': {'dimensions': {'column': '1'},
                                              'name': 'integer_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             '0f0ab90137aa': {'dimensions': {'column': '2'},
                                              'distributionValue': {'sketchImpl': 'KLL10'},
                                              'name': 'distribution',
                                              'type': 'DISTRIBUTION'},
                             '1f3c3ac80b3a': {'dimensions': {'column': '0'},
                                              'name': 'missing_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             '32143615ee5a': {'dimensions': {'column': '1'},
                                              'distributionValue': {'sketchImpl': 'KLL10'},
                                              'name': 'distribution',
                                              'type': 'DISTRIBUTION'},
                             '41ed21ff8956': {'dimensions': {'column': '2'},
                                              'name': 'float_values',
                                              'ratioValue': {'counter': 1.0, 'total': 1.0},
                                              'type': 'RATIO'},
                             '5c917c4a138c': {'dimensions': {'column': '3'},
                                              'name': 'zero_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             '685542684133': {'dimensions': {'column': '0'},
                                              'name': 'integer_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             '73c707ce113c': {'dimensions': {'column': '2'},
                                              'name': 'zero_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             '785cfd27ee94': {'dimensions': {'column': '3'},
                                              'distributionValue': {'sketchImpl': 'KLL10'},
                                              'name': 'distribution',
                                              'type': 'DISTRIBUTION'},
                             '81dcca1fbb5c': {'dimensions': {'column': '2'},
                                              'name': 'missing_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             '8e1bc8013472': {'counterValue': {'counter': 1.0},
                                              'name': 'instance_count',
                                              'type': 'COUNTER'},
                             '903e0399f098': {'dimensions': {'column': '3'},
                                              'name': 'float_values',
                                              'ratioValue': {'counter': 1.0, 'total': 1.0},
                                              'type': 'RATIO'},
                             'c07ffa6709a7': {'dimensions': {'column': '2'},
                                              'name': 'integer_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             'c20d28d3bc8e': {'dimensions': {'column': '1'},
                                              'name': 'zero_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             'dd7593fd9cae': {'dimensions': {'column': '0'},
                                              'name': 'float_values',
                                              'ratioValue': {'counter': 1.0, 'total': 1.0},
                                              'type': 'RATIO'},
                             'de5757c0d4d2': {'dimensions': {'column': '1'},
                                              'name': 'missing_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             'e3f8dbe7ba73': {'dimensions': {'column': '3'},
                                              'name': 'integer_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'},
                             'ee6941f991dd': {'dimensions': {'column': '0'},
                                              'distributionValue': {'sketchImpl': 'KLL10'},
                                              'name': 'distribution',
                                              'type': 'DISTRIBUTION'},
                             'f038d97c5e5a': {'dimensions': {'column': '0'},
                                              'name': 'zero_values',
                                              'ratioValue': {'total': 1.0},
                                              'type': 'RATIO'}})

    def test_update_metrics_perf(self):
        window = metrics_pb2.PredictionWindow()
        metric_updaters = {}

        prediction_buffer = []
        for i in range(100):
            input_data = {}
            for j in range(50):
                input_data['f1_' + str(j)] = i
                input_data['f2_' + str(j)] = i + 3.53242342
                input_data['f3_' + str(j)] = 100 * 'abc' + str(i)
                input_data['f4_' + str(j)] = float('nan')
                input_data['f4_' + str(j)] = float('+inf')
            output_data = [0.1, 0.2, 0.1, 0.4]
            prediction_buffer.append(
                Prediction(
                    input_data=input_data,
                    output_data=output_data))

        #import cProfile
        #from pstats import Stats, SortKey

        start_ts = time.time()

        # with cProfile.Profile() as pr:
        for k in range(10):
            statistics.update_metrics(
                metric_updaters, window, prediction_buffer)
        #stats = Stats(pr)
        # stats.sort_stats(SortKey.CUMULATIVE).print_stats(25)

        for metric_updater in metric_updaters.values():
            metric_updater.finalize()

        took = time.time() - start_ts

        print('update_metrics took (sec): ', round(took, 6))
        self.assertTrue(len(window.data_streams[str(
            metrics_pb2.DataStream.DataSource.MODEL_INPUT)].metrics) > 0)
        self.assertTrue(len(window.data_streams[str(
            metrics_pb2.DataStream.DataSource.MODEL_OUTPUT)].metrics) > 0)
        self.assertTrue(took < 1)

    def test_truncate_strings(self):
        truncated = statistics._truncate_strings(
            ['abc', '1234567890abcdefgh', '1234567890abcdefghi'])
        self.assertEqual(
            truncated, [
                'abc', '1234567890abcdefgh', '1234567890...efghi'])
